#!/usr/bin/python
#rsvp4gp2x - Rapid Serial Visual Projection reader for the GP2X
#Copyright (C) 2007  Kevin Granade

#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#initialize python modules

#just load the modules that I use instead of the entire pygame module
import sys
import os
#import pygame
import zlib
import pickle
import getopt
import rsvp_functions

from rsvp_functions import *

#control state, encapsulated in an object for easy passing
#filename is a mandatory argument since I just don't seem to "get" python strings
class app_state:
  def __init__(self, filename):
    self.file_name_string = filename
    self.delay = 200
    self.iterate_foward = True
    self.pause = False
    self.bookmarks = [0]
    self.index = 0
    self.rotation = 0
    self.fast_foward = 0
    self.text_size = 50
    self.text_color = (255,150,0)
    self.delay_size = 50
    self.delay_color = (255,150,0)
    self.progressbar_color = (0,255,0)
    self.bookmark_color = (255,0,0)
    self.background_color = (0,0,0)
    self.word_skip = 10
    self.comma_delay = 10
    self.period_delay = 20
    self.rate_display = get_delay
    self.skip_words = 0

zipped = False
text_color = [255,150,0]
background_color = [0,0,0]

#process arguments
try:
  opts, args = getopt.getopt(sys.argv[1:], "ht:z:", ["help", "text_file=", "zipped_file="])
except getopt.GetoptError:
  usage()
  sys.exit(2)

for opt, arg in opts:
  if opt in ("-h", "--help"):
    usage()
    sys.exit()
  elif opt in ("-t", "--text_file"):
    current_state = app_state(arg)
    bool_filename = True
  elif opt in ("-z", "--zipped_file"):
    current_state = app_state(arg)
    zipped = True
    bool_filename = True

do_config(current_state)

#initialize libraries
#can't figure out how to init time without initing the whole shebang
pygame.init()
#pygame.joystick.init()
#pygame.font.init()
#pygame.display.init()

#init joystick, this also provides a loose test as to wether we are running
#on a gp2x, on many pcs there will be no joystick, and if there is no joystick,
#we need to fall back to a keyboard anyway.  The problem is if we are on a pc
# and a joystick is detected, whatever
num_joysticks = pygame.joystick.get_count()
if num_joysticks > 0:
    stick = pygame.joystick.Joystick(0)
    stick.init() # now we will receive events for the joystick
    #set do_events to the function we will be using
    do_events = do_events_modular
else:
    do_events = do_events_keyboard

#hide the mouse cursor
pygame.mouse.set_visible(False)

#set size of display
h_screen_size = 320
v_screen_size = 240

#open a file and process it into a text string
#unhandled exceptions are fine here, we can't support not opening the file
#though we might want to report an error
input_file = open(current_state.file_name_string)
if zipped:
  zipped_string = input_file.read()
  text_string = zlib.decompress(zipped_string)
else:
  text_string = input_file.read()
input_file.close()

#open bookmark file, if it exists
try:
  bookmark_file = open(".".join([current_state.file_name_string, "bookmark"]))
  try:
    try:
      current_state.bookmarks = pickle.load(bookmark_file)
    except EOFError: pass #ignore EOF errors, if the file is empty we just don't init the bookmarks
  finally:
    bookmark_file.close()
except IOError: pass

#create fonts
text_font = pygame.font.SysFont(None, current_state.text_size)
delay_font = pygame.font.SysFont(None, current_state.delay_size)

#change string into a list of strings, each of which is a word
#may or may not want to optimize by removing string after splitting
text_list = text_string.split()

#grab screen, need to see if this is set as it should be, does pygame for gp2x support hw surfaces?
main_surface = pygame.display.set_mode((h_screen_size, v_screen_size), pygame.FULLSCREEN)

#while text exists
#initialize "old" rect to nothing, will probably be initializing index to a bookmark in the future
old_dirty_rect = pygame.Rect((0,0),(0,0))
old_delay_display_rect = ((0,0),(150,50))
progress_bar_rect = pygame.Rect((0,v_screen_size - 10),(h_screen_size,10))
text_list_length = len(text_list)
while True:
  old_time = pygame.time.get_ticks()
  a_word = text_list[current_state.index]
  #check for commands
  do_events(current_state)

  #render delay reading
  rate_string = current_state.rate_display(current_state)
  rendered_delay = delay_font.render(str(rate_string), True, current_state.delay_color, current_state.background_color)
  pygame.draw.rect(main_surface, [0,0,0], old_delay_display_rect)

  #render text
  rendered_text = text_font.render(a_word, True, current_state.text_color, current_state.background_color)

  #blit text to screen
  pygame.draw.rect(main_surface, [0,0,0], old_dirty_rect)
  if current_state.rotation != 0:
    rendered_text = pygame.transform.rotate(rendered_text, current_state.rotation)
    rendered_delay = pygame.transform.rotate(rendered_delay, current_state.rotation)

  #get a centered text offset, might try replacing this with:
  #  h_offset, v_offset = rendered_text.get_size() 
  #or inserting it into the blit call (only if I can divide them by two at the same time)
  text_render_size = rendered_text.get_size()
  h_offset = h_screen_size - text_render_size[0] >> 1
  v_offset = v_screen_size - text_render_size[1] >> 1
  new_dirty_rect = main_surface.blit(rendered_text, [h_offset,v_offset])
  new_delay_display_rect = main_surface.blit(rendered_delay, [0,0])

  #draw progress bar and bookmarks, might not want to redraw every cycle
  #might want to pre-calculate h_screen_size* mark when bookmarks are unpickled and inserted
  pygame.draw.rect(main_surface, [0,0,0], progress_bar_rect)
  for mark in current_state.bookmarks:
    pygame.draw.rect(main_surface, current_state.bookmark_color, ((h_screen_size*mark)/text_list_length,v_screen_size - 10,1,5))
  pygame.draw.rect(main_surface, current_state.progressbar_color, (0,v_screen_size - 5,(h_screen_size*current_state.index)/text_list_length,5))

  pygame.display.update([old_dirty_rect,new_dirty_rect,old_delay_display_rect,new_delay_display_rect,progress_bar_rect])

  old_dirty_rect = new_dirty_rect
  old_delay_display_rect = new_delay_display_rect

  render_time = pygame.time.get_ticks() - old_time
  pygame.time.wait(current_state.delay - render_time - (current_state.fast_foward * 100))

  if not current_state.pause:
    if current_state.skip_words != 0:
      current_state.index += current_state.skip_words*current_state.word_skip
      current_state.skip_words = 0
    elif current_state.iterate_foward:
      if current_state.fast_foward:
        current_state.index += current_state.word_skip
      else:
        current_state.index += 1
    else:
      current_state.index -= 1
    if current_state.index < 0:
      current_state.index = 0
      toggle_pause(current_state)
      continue
    elif current_state.index >= text_list_length:
      current_state.index = text_list_length - 1
      toggle_pause(current_state)
      continue

clean_exit(current_state)
